#include <SFML/Window/Event.hpp>
#include "application.h"

constexpr int SCREEN_WIDTH   = 1280;
constexpr int SCREEN_HEIGHT  = 720;
const sf::Time TIME_PER_SECOND = sf::seconds(1.f / 60.f);

Application::Application() :
    render_window({SCREEN_WIDTH, SCREEN_HEIGHT}, "Sliding Puzzle")
{}

void Application::run()
{
    render_window.setFramerateLimit(60);
    render_window.display();

    while (render_window.isOpen())
    {
        processInput();
        draw();
    }
}

void Application::draw()
{
    render_window.clear();
    board.draw(render_window);
    render_window.setView(render_window.getDefaultView());
    render_window.display();
}

bool Application::processInput()
{
    sf::Event event;
    while (render_window.pollEvent(event))
    {
        switch (event.type)
        {
        case sf::Event::Closed:
            render_window.close();
            break;

        case sf::Event::KeyPressed:
            if (board.isWinCondition())
                return true;
            if (event.key.code == sf::Keyboard::Z)
                board.onSelectionMode();
            else
                board.moveSelection(getDirection(event.key.code));
            break;

        default:
            break;
        }
    }

    return true;
}

DIRECTION Application::getDirection(sf::Keyboard::Key &key) const
{
    switch (key)
    {
    case sf::Keyboard::A:
    case sf::Keyboard::Left:
    case sf::Keyboard::Num4:
        return DIRECTION::LEFT;

    case sf::Keyboard::W:
    case sf::Keyboard::Up:
    case sf::Keyboard::Num8:
        return DIRECTION::UP;

    case sf::Keyboard::D:
    case sf::Keyboard::Right:
    case sf::Keyboard::Num6:
        return DIRECTION::RIGHT;

    case sf::Keyboard::S:
    case sf::Keyboard::Down:
    case sf::Keyboard::Num2:
        return DIRECTION::DOWN;

    default:
        return DIRECTION::NONE;
    }
}
